@@ -40,11 +40,36 @@ class WeConnectSession(VWWebSession):
             'content-type': 'application/json',
             'content-version': '1',
             'x-newrelic-id': 'VgAEWV9QDRAEXFlRAAYPUA==',
-            'user-agent': 'WeConnect/5 CFNetwork/1206 Darwin/20.1.0',
+            'user-agent': 'WeConnect/3 CFNetwork/1331.0.7 Darwin/21.4.0',
             'accept-language': 'de-de',
         })
 
+    def request(
+        self,
+        method,
+        url,
+        data=None,
+        headers=None,
+        withhold_token=False,
+        access_type=AccessType.ACCESS,
+        token=None,
+        timeout=None,
+        **kwargs
+    ):
+        """Intercept all requests and add weconnect-trace-id header."""
+
+        import secrets
+        traceId = secrets.token_hex(16)
+        weConnectTraceId = (traceId[:8] + '-' + traceId[8:12] + '-' + traceId[12:16] + '-' + traceId[16:20] + '-' + traceId[20:]).upper()
+        headers = headers or {}
+        headers['weconnect-trace-id'] = weConnectTraceId
+
+        return super(WeConnectSession, self).request(
+            method, url, headers=headers, data=data, withhold_token=withhold_token, access_type=access_type, token=token, timeout=timeout, **kwargs
+        )
+
     def login(self):
+        super(WeConnectSession, self).login()
         authorizationUrl = self.authorizationUrl(url='https://identity.vwgroup.io/oidc/v1/authorize')
         response = self.doWebAuth(authorizationUrl)
         self.fetchTokens('https://login.apps.emea.vwapps.io/login/v1',
@@ -95,7 +120,16 @@ class WeConnectSession(VWWebSession):
             loginFormResponse: requests.Response = websession.get(authorizationUrl, allow_redirects=False)
             if loginFormResponse.status_code == requests.codes['ok']:
                 break
-            authorizationUrl = loginFormResponse.headers['Location']
+            elif loginFormResponse.status_code == requests.codes['found']:
+                if 'Location' in loginFormResponse.headers:
+                    authorizationUrl = loginFormResponse.headers['Location']
+                else:
+                    raise APICompatibilityError('Forwarding without Location in Header')
+            elif loginFormResponse.status_code == requests.codes['internal_server_error']:
+                raise RetrievalError('Temporary server error during login')
+            else:
+                raise APICompatibilityError('Retrieving credentials page was not successfull,'
+                                            f' status code: {loginFormResponse.status_code}')
 
         # Find login form on page to obtain inputs
         emailFormRegex = r'<form.+id=\"emailPasswordForm\".*action=\"(?P<formAction>[^\"]+)\"[^>]*>' \
@@ -134,7 +168,7 @@ class WeConnectSession(VWWebSession):
                                         f' status code: {login2Response.status_code}')
 
         credentialsTemplateRegex = r'<script>\s+window\._IDK\s+=\s+\{\s' \
-            r'(?P<templateModel>.+?(?=\s+\};\s+</script>))\s+\};\s+</script>'
+            r'(?P<templateModel>.+?(?=\s+\};?\s+</script>))\s+\};?\s+</script>'
         match = re.search(credentialsTemplateRegex, login2Response.text, flags=re.DOTALL)
         if match is None:
             raise APICompatibilityError('No credentials form found')
@@ -153,6 +187,12 @@ class WeConnectSession(VWWebSession):
                         form2Data['hmac'] = templateModel['hmac']
                     if 'emailPasswordForm' in templateModel and 'email' in templateModel['emailPasswordForm']:
                         form2Data['email'] = templateModel['emailPasswordForm']['email']
+                    if 'error' in templateModel and templateModel['error'] is not None:
+                        if templateModel['error'] == 'validator.email.invalid':
+                            raise AuthentificationError('Error during login, email invalid')
+                        raise AuthentificationError(f'Error during login: {templateModel["error"]}')
+                    if 'registerCredentialsPath' in templateModel and templateModel['registerCredentialsPath'] == 'register':
+                        raise AuthentificationError(f'Error during login, account {self.sessionuser.username} does not exist')
                     if 'errorCode' in templateModel:
                         raise AuthentificationError('Error during login, is the username correct?')
                     if 'postAction' in templateModel:
@@ -208,6 +248,8 @@ class WeConnectSession(VWWebSession):
             if 'consent' in afterLoginUrl:
                 consentURL = afterLoginUrl
             afterLoginResponse = self.get(afterLoginUrl, allow_redirects=False, access_type=AccessType.NONE)
+            if afterLoginResponse.status_code == requests.codes['internal_server_error']:
+                raise RetrievalError('Temporary server error during login')
 
             if 'Location' not in afterLoginResponse.headers:
                 if consentURL is not None:
@@ -249,12 +291,17 @@ class WeConnectSession(VWWebSession):
             loginHeadersForm['accept'] = 'application/json'
 
             tokenResponse = self.post(token_url, headers=loginHeadersForm, data=body, allow_redirects=False, access_type=AccessType.ID)
+            if tokenResponse.status_code != requests.codes['ok']:
+                raise TemporaryAuthentificationError(f'Token could not be fetched due to temporary WeConnect failure: {tokenResponse.status_code}')
             token = self.parseFromBody(tokenResponse.text)
 
             return token
 
     def parseFromBody(self, token_response, state=None):
-        token = json.loads(token_response)
+        try:
+            token = json.loads(token_response)
+        except json.decoder.JSONDecodeError:
+            raise TemporaryAuthentificationError('Token could not be refreshed due to temporary WeConnect failure: json could not be decoded')
         if 'accessToken' in token:
             token['access_token'] = token.pop('accessToken')
         if 'idToken' in token:
