@@ -21,6 +21,7 @@ from weconnect.elements.controls import Controls
 from weconnect.elements.access_status import AccessStatus
 from weconnect.elements.battery_status import BatteryStatus
 from weconnect.elements.lv_battery_status import LVBatteryStatus
+from weconnect.elements.battery_support_status import BatterySupportStatus
 from weconnect.elements.capability_status import CapabilityStatus
 from weconnect.elements.charging_status import ChargingStatus
 from weconnect.elements.charging_settings import ChargingSettings
@@ -28,6 +29,7 @@ from weconnect.elements.charge_mode import ChargeMode
 from weconnect.elements.climatization_status import ClimatizationStatus
 from weconnect.elements.climatization_settings import ClimatizationSettings
 from weconnect.elements.climatization_timer import ClimatizationTimer
+from weconnect.elements.departure_timers_status import DepartureTimersStatus
 from weconnect.elements.lights_status import LightsStatus
 from weconnect.elements.maintenance_status import MaintenanceStatus
 from weconnect.elements.warning_lights_status import WarningLightsStatus
@@ -43,12 +45,13 @@ from weconnect.errors import APICompatibilityError, RetrievalError, APIError
 from weconnect.util import toBool
 from weconnect.weconnect_errors import ErrorEventType
 from weconnect.domain import Domain
+from weconnect.elements.error import Error
 
 from weconnect.elements.helpers.request_tracker import RequestTracker
 
 SUPPORT_IMAGES = False
 try:
-    from PIL import Image  # type: ignore
+    from PIL import Image, ImageDraw  # type: ignore
     SUPPORT_IMAGES = True
 except ImportError:
     pass
@@ -56,6 +59,21 @@ except ImportError:
 LOG: logging.Logger = logging.getLogger("weconnect")
 
 
+class DomainDict(AddressableDict):
+    def __init__(self, **kwargs):
+        self.error: Error = Error(localAddress='error', parent=self)
+        super(DomainDict, self).__init__(**kwargs)
+
+    def updateError(self, fromDict: Dict[str, Any]):
+        if 'error' in fromDict:
+            self.error.update(fromDict['error'])
+        else:
+            self.error.reset()
+
+    def hasError(self) -> bool:
+        return self.error.enabled
+
+
 class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attributes
 
     def __init__(
@@ -85,8 +103,9 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                                                                                                  value=None,
                                                                                                  valueType=Vehicle.DevicePlatform)
         self.nickname: AddressableAttribute[str] = AddressableAttribute(localAddress='nickname', parent=self, value=None, valueType=str)
+        self.brandCode: AddressableAttribute[str] = AddressableAttribute(localAddress='brandCode', parent=self, value=None, valueType=Vehicle.BrandCode)
         self.capabilities: AddressableDict[str, GenericCapability] = AddressableDict(localAddress='capabilities', parent=self)
-        self.domains: AddressableDict[str, AddressableDict[str, GenericStatus]] = AddressableDict(localAddress='domains', parent=self)
+        self.domains: AddressableDict[str, DomainDict[str, GenericStatus]] = AddressableDict(localAddress='domains', parent=self)
         self.images: AddressableAttribute[Dict[str, str]] = AddressableAttribute(localAddress='images', parent=self, value=None, valueType=dict)
         self.tags: AddressableAttribute[List[str]] = AddressableAttribute(localAddress='tags', parent=self, value=None, valueType=list)
         self.coUsers: AddressableList[Vehicle.User] = AddressableList(localAddress='coUsers', parent=self)
@@ -135,6 +154,7 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
             self.model.fromDict(fromDict, 'model')
             self.devicePlatform.fromDict(fromDict, 'devicePlatform')
             self.nickname.fromDict(fromDict, 'nickname')
+            self.brandCode.fromDict(fromDict, 'brandCode')
 
             if updateCapabilities and 'capabilities' in fromDict and fromDict['capabilities'] is not None:
                 for capDict in fromDict['capabilities']:
@@ -188,6 +208,7 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                                               'model',
                                               'devicePlatform',
                                               'nickname',
+                                              'brandCode',
                                               'capabilities',
                                               'images',
                                               'tags',
@@ -227,6 +248,9 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                 'chargingSettingsRequestStatus': GenericRequestStatus,
                 'chargingCareSettings': GenericSettings,
             },
+            Domain.CHARGING_PROFILES: {
+                'chargingProfilesStatus': ChargingProfiles,
+            },
             Domain.CLIMATISATION: {
                 'climatisationStatus': ClimatizationStatus,
                 'climatisationSettings': ClimatizationSettings,
@@ -234,6 +258,9 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                 'climatisationRequestStatus': GenericRequestStatus,
                 'climatisationSettingsRequestStatus': GenericRequestStatus,
             },
+            Domain.DEPARTURE_TIMERS: {
+                'departureTimersStatus': DepartureTimersStatus,
+            },
             Domain.FUEL_STATUS: {
                 'rangeStatus': RangeStatus,
             },
@@ -263,12 +290,24 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                 'measurements': GenericStatus,
             },
             Domain.BATTERY_SUPPORT: {
-                'batterySupportStatus': GenericStatus,
+                'batterySupportStatus': BatterySupportStatus,
             }
         }
         if self.vin.value is None:
             raise APIError('')
-        if selective is None or Domain.ALL in selective:
+        if selective is None:
+            jobs = [domain.value for domain in Domain if domain != Domain.ALL and domain != Domain.ALL_CAPABLE and domain != Domain.PARKING]
+        elif Domain.ALL_CAPABLE in selective:
+            if self.capabilities:
+                jobs = []
+                for dom in [domain for domain in Domain if domain != Domain.ALL and domain != Domain.ALL_CAPABLE and domain != Domain.PARKING]:
+                    if dom.value in self.capabilities and self.capabilities[dom.value].enabled and not self.capabilities[dom.value].status.enabled:
+                        jobs.append(dom.value)
+                if updateCapabilities:
+                    jobs.append(Domain.USER_CAPABILITIES.value)
+            else:
+                jobs = ['all']
+        elif Domain.ALL in selective:
             jobs = ['all']
         else:
             jobs = [domain.value for domain in selective]
@@ -280,7 +319,7 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                     continue
                 if domain.value in data:
                     if domain.value not in self.domains:
-                        self.domains[domain.value] = AddressableDict(localAddress=domain.value, parent=self.domains)
+                        self.domains[domain.value] = DomainDict(localAddress=domain.value, parent=self.domains)
                     for key, className in keyClassMap.items():
                         if key in data[domain.value]:
                             if key in self.domains[domain.value]:
@@ -290,10 +329,13 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                                 LOG.debug('Status %s does not exist, creating it', key)
                                 self.domains[domain.value][key] = className(vehicle=self, parent=self.domains[domain.value], statusId=key,
                                                                             fromDict=data[domain.value][key], fixAPI=self.fixAPI)
+                    if 'error' in data[domain.value]:
+                        self.domains[domain.value].updateError(data[domain.value])
 
                     # check that there is no additional status than the configured ones, except for "target" that we merge into
                     # the known ones
-                    for key, value in {key: value for key, value in data[domain.value].items() if key not in list(keyClassMap.keys())}.items():
+                    for key, value in {key: value for key, value in data[domain.value].items()
+                                       if key not in list(keyClassMap.keys()) and key not in ['error']}.items():
                         LOG.warning('%s: Unknown attribute %s with value %s in domain %s', self.getGlobalAddress(), key, value, domain.value)
             # check that there is no additional domain than the configured ones
             for key, value in {key: value for key, value in data.items() if key not in list([domain.value for domain in jobKeyClassMap.keys()])}.items():
@@ -302,7 +344,7 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
         # Controls
         self.controls.update()
 
-        if (selective is None or any(x in selective for x in [Domain.ALL, Domain.PARKING])) \
+        if (selective is None or any(x in selective for x in [Domain.ALL, Domain.ALL_CAPABLE, Domain.PARKING])) \
                 and (not updateCapabilities or ('parkingPosition' in self.capabilities and self.capabilities['parkingPosition'].status.value is None)):
             url = 'https://mobileapi.apps.emea.vwapps.io/vehicles/' + self.vin.value + '/parkingposition'
             data = self.weConnect.fetchData(url, force, allowEmpty=True, allowHttpError=True, allowedErrors=[codes['not_found'],
@@ -312,7 +354,7 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
 
             if data is not None:
                 if 'parking' not in self.domains:
-                    self.domains['parking'] = AddressableDict(localAddress='parking', parent=self)
+                    self.domains['parking'] = DomainDict(localAddress='parking', parent=self)
                 if 'parkingPosition' in self.domains['parking']:
                     self.domains['parking']['parkingPosition'].update(fromDict=data)
                 else:
@@ -412,7 +454,16 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
 
             badges: Set[Vehicle.Badge] = set()
 
-            if 'access' in self.domains and 'accessStatus' in self.domains['access']:
+            doorNameMap: Dict[str, str] = {'frontLeft': 'door_left_front',
+                                           'frontRight': 'door_right_front',
+                                           'rearLeft': 'door_left_back',
+                                           'rearRight': 'door_right_back'}
+            windowNameMap: Dict[str, str] = {'frontLeft': 'window_left_front',
+                                             'frontRight': 'window_right_front',
+                                             'rearLeft': 'window_left_back',
+                                             'rearRight': 'window_right_back',
+                                             'sunRoof': 'sunroof'}
+            if 'access' in self.domains and 'accessStatus' in self.domains['access'] and not self.domains['access']['accessStatus'].error.enabled:
                 accessStatus: AccessStatus = cast(AccessStatus, self.domains['access']['accessStatus'])
 
                 if accessStatus.overallStatus.enabled:
@@ -424,10 +475,6 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                         badges.add(Vehicle.Badge.WARNING)
 
                 for name, door in accessStatus.doors.items():
-                    doorNameMap: Dict[str, str] = {'frontLeft': 'door_left_front',
-                                                   'frontRight': 'door_right_front',
-                                                   'rearLeft': 'door_left_back',
-                                                   'rearRight': 'door_right_back'}
                     name = doorNameMap.get(name, name)
                     doorImageName: Optional[str] = None
 
@@ -444,11 +491,6 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                         img.paste(doorImage, (0, 0), doorImage)
 
                 for name, window in accessStatus.windows.items():
-                    windowNameMap: Dict[str, str] = {'frontLeft': 'window_left_front',
-                                                     'frontRight': 'window_right_front',
-                                                     'rearLeft': 'window_left_back',
-                                                     'rearRight': 'window_right_back',
-                                                     'sunRoof': 'sunroof'}
                     name = windowNameMap.get(name, name)
                     windowImageName: Optional[str] = None
 
@@ -463,6 +505,15 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                     if windowImageName is not None and windowImageName in self.__carImages:
                         windowImage = self.__carImages[windowImageName].convert("RGBA")
                         img.paste(windowImage, (0, 0), windowImage)
+            else:
+                for name in doorNameMap.values():
+                    if name in self.__carImages:
+                        doorImage = self.__carImages[name].convert("RGBA")
+                        img.paste(doorImage, (0, 0), doorImage)
+                for name in windowNameMap.values():
+                    if name != 'sunroof' and name in self.__carImages:
+                        windowImage = self.__carImages[name].convert("RGBA")
+                        img.paste(windowImage, (0, 0), windowImage)
 
             if 'vehicleLights' in self.domains and 'lightsStatus' in self.domains['vehicleLights']:
                 lightsStatus: LightsStatus = cast(LightsStatus, self.domains['vehicleLights']['lightsStatus'])
@@ -482,8 +533,12 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
 
             if 'charging' in self.domains and 'chargingStatus' in self.domains['charging']:
                 chargingStatus: ChargingStatus = cast(ChargingStatus, self.domains['charging']['chargingStatus'])
-                if chargingStatus.chargingState.value == ChargingStatus.ChargingState.CHARGING:
+                if chargingStatus.chargingState.value in (ChargingStatus.ChargingState.CHARGING,
+                                                          ChargingStatus.ChargingState.CHARGE_PURPOSE_REACHED_CONSERVATION,
+                                                          ChargingStatus.ChargingState.CONSERVATION):
                     badges.add(Vehicle.Badge.CHARGING)
+                elif chargingStatus.chargingState.value == ChargingStatus.ChargingState.ERROR:
+                    badges.add(Vehicle.Badge.WARNING)
 
             if 'charging' in self.domains and 'plugStatus' in self.domains['charging']:
                 plugStatus: PlugStatus = cast(PlugStatus, self.domains['charging']['plugStatus'])
@@ -513,6 +568,20 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                 imgWithBadges.paste(badgeImage, (0, badgeoffset), badgeImage)
                 badgeoffset += 110
 
+            warningLightoffset = 0
+            imgWidth, _ = imgWithBadges.size
+            if 'vehicleHealthWarnings' in self.domains and 'warningLights' in self.domains['vehicleHealthWarnings']:
+                warningLightsStatus = self.domains['vehicleHealthWarnings']['warningLights']
+                if warningLightsStatus.warningLights.enabled:
+                    for warningLight in warningLightsStatus.warningLights.values():
+                        if warningLight.icon.enabled:
+                            draw = ImageDraw.Draw(imgWithBadges)
+                            draw.ellipse(((imgWidth - 100), warningLightoffset, (imgWidth - 1), (warningLightoffset + 100)), fill=(0, 0, 0, 200))
+                            lightImage = warningLight.icon.value
+                            lightImage = lightImage.resize((64, 64), Image.ANTIALIAS)
+                            imgWithBadges.paste(lightImage, ((imgWidth - 82), warningLightoffset + 18), lightImage)
+                            warningLightoffset += 110
+
             self.__carImages['statusWithBadge'] = imgWithBadges
 
             # Car with badges
@@ -525,6 +594,20 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                     carWithBadges.paste(badgeImage, (0, badgeoffset), badgeImage)
                     badgeoffset += 110
 
+                warningLightoffset = 0
+                imgWidth, _ = carWithBadges.size
+                if 'vehicleHealthWarnings' in self.domains and 'warningLights' in self.domains['vehicleHealthWarnings']:
+                    warningLightsStatus = self.domains['vehicleHealthWarnings']['warningLights']
+                    if warningLightsStatus.warningLights.enabled:
+                        for warningLight in warningLightsStatus.warningLights.values():
+                            if warningLight.icon.enabled:
+                                draw = ImageDraw.Draw(carWithBadges)
+                                draw.ellipse(((imgWidth - 100), warningLightoffset, (imgWidth - 1), (warningLightoffset + 100)), fill=(0, 0, 0, 200))
+                                lightImage = warningLight.icon.value
+                                lightImage = lightImage.resize((64, 64), Image.ANTIALIAS)
+                                carWithBadges.paste(lightImage, ((imgWidth - 82), warningLightoffset + 18), lightImage)
+                                warningLightoffset += 110
+
                 self.__carImages['carWithBadge'] = carWithBadges
 
         else:
@@ -577,6 +660,8 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
             returnString += f'Device Platform:   {self.devicePlatform.value.value}\n'
         if self.nickname.enabled and self.nickname.value is not None:
             returnString += f'Nickname:          {self.nickname.value}\n'
+        if self.brandCode.enabled and self.brandCode.value is not None:
+            returnString += f'Brand Code:        {self.brandCode.value.value}\n'
         if self.role.enabled and self.role.value is not None:
             returnString += f'Role:              {self.role.value.value}\n'  # pylint: disable=no-member
         if self.enrollmentStatus.enabled and self.enrollmentStatus.value is not None:
@@ -602,6 +687,8 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                 for status in self.domains[domain].values():
                     if status.enabled:
                         returnString += ''.join(['\t' + line for line in str(status).splitlines(True)]) + '\n'
+                if self.domains[domain].hasError():
+                    returnString += ''.join(['\t' + line for line in f'Error: {self.domains[domain].error}'.splitlines(True)]) + '\n'
         return returnString
 
     class Badge(Enum):
@@ -618,9 +705,14 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
     class DevicePlatform(Enum,):
         MBB = 'MBB'
         MBB_ODP = 'MBB_ODP'
+        MBB_OFFLINE = 'MBB_OFFLINE'
         WCAR = 'WCAR'
         UNKNOWN = 'UNKNOWN'
 
+    class BrandCode(Enum,):
+        V = 'V'
+        UNKNOWN = 'unknown brand code'
+
     class User(AddressableObject):
         def __init__(
             self,
@@ -687,6 +779,7 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
             PRIMARY_USER = 'PRIMARY_USER'
             SECONDARY_USER = 'SECONDARY_USER'
             GUEST_USER = 'GUEST_USER'
+            CDIS_UNKNOWN_USER = 'CDIS_UNKNOWN_USER'
             UNKNOWN = 'UNKNOWN'
 
             def __str__(self) -> str:
@@ -702,4 +795,8 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
 
         class UserRoleStatus(Enum,):
             ENABLED = 'ENABLED'
+            DISABLED_HMI = 'DISABLED_HMI'
+            DISABLED_SPIN = 'DISABLED_SPIN'
+            DISABLED_PU_SPIN_RESET = 'DISABLED_PU_SPIN_RESET'
+            CDIS_UNKNOWN_USER = 'CDIS_UNKNOWN_USER'
             UNKNOWN = 'UNKNOWN'
